/**    
  ******************************************************************************
  * @attention    
  * Copyright 2023 SHANGHAI FUDAN MICROELECTRONICS GROUP CO., LTD.(FUDAN MICRO.)
  *        
  * Redistribution and use in source and binary forms, with or without modification,
  * are permitted provided that the following conditions are met: 
  *    
  * 1. Redistributions of source code must retain the above copyright notice, 
  *    this list of conditions and the following disclaimer.
  *    
  * 2. Redistributions in binary form must reproduce the above copyright notice,
  *    this list of conditions and the following disclaimer in the documentation
  *    and/or other materials provided with the distribution.
  *    
  * 3. Neither the name of the copyright holder nor the names of its contributors 
  *    may be used to endorse or promote products derived from this software without
  *    specific prior written permission.
  *    
  * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS “AS IS”
  * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE   
  * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
  * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
  * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
  * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
  * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT   
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
  *******************************************************************************
  */
#include "main.h"
#include "user_init.h"
#include "uart.h"
#include "lin.h"
#include "sleep.h"
#include "svd_interrupt.h"
#include "key.h"
#include "adc.h"
#include "gptim.h"
#include "i2c.h"
#include "spi.h"
#include "clock.h"
#include "verify.h"
#include "led.h"

#if (SLEEP_SUPPORT == 1)
uint8_t  g_awake_init = 0;

uint8_t  g_master_awake_init = 0;   /* 0-无sleep 1-sleep 2-awake */
uint8_t  g_slave_awake_init = 0;   /* 0-无sleep 1-sleep 2-awake */

/**
  * @brief  LIN RX引脚GPIO中断初始化
  * @param  void
  * @retval void
  */
void GPIO_interrupt_init(void)
{
    FL_GPIO_InitTypeDef GPIO_InitStruct = {0};

    /* 引脚边沿触发选择 */
    FL_RCC_EnableEXTIOnSleep();                                 /* 休眠使能外部中断采样 */
    FL_RCC_EnableGroup1OperationClock(FL_RCC_GROUP1_OPCLK_EXTI);/* EXTI工作时钟使能 */
    
    /* EXTI中断采样时钟选择 */
    FL_RCC_SetEXTIClockSource(FL_RCC_EXTI_CLK_SOURCE_LSCLK);

    /* 用到的GPIO引脚 设为输入 */
    GPIO_InitStruct.pin        = LIN_UART_RX_PIN;
    GPIO_InitStruct.mode       = FL_GPIO_MODE_INPUT;
    GPIO_InitStruct.outputType = FL_GPIO_OUTPUT_PUSHPULL;
    GPIO_InitStruct.pull       = FL_ENABLE;
    GPIO_InitStruct.remapPin   = FL_DISABLE;
    FL_GPIO_Init(LIN_UART_RX_GPIO, &GPIO_InitStruct);

    FL_GPIO_SetTriggerEdge(GPIO, LIN_RX_EXTI_LINE, FL_GPIO_EXTI_TRIGGER_EDGE_DISABLE); /* 关闭边沿触发 */

    /* 每条LINE只能选择一个引脚 */
    FL_GPIO_SetExtiLine4(GPIO, LIN_RX_EXTI_LINE_PIN);           /* 中断引脚选择 */
    FL_GPIO_ClearFlag_EXTI(GPIO, LIN_RX_EXTI_LINE);             /* 清除中断标志 */
    NVIC_ClearPendingIRQ(GPIO_IRQn);
    FL_GPIO_SetTriggerEdge(GPIO, LIN_RX_EXTI_LINE, FL_GPIO_EXTI_TRIGGER_EDGE_FALLING); /* 边沿选择 */

    /* NVIC中断配置 */
    NVIC_DisableIRQ(GPIO_IRQn);
    NVIC_SetPriority(GPIO_IRQn, LIN_GPIO_IRQ_PRI); 
    NVIC_EnableIRQ(GPIO_IRQn);
}

/**
  * @brief  外设 Sleep切换
  * @param  void
  * @retval void
  */
void prepare_sleep(void)
{
    Close_None_GPIO();
    
    /* LIN 休眠初始化配置 */
    LIN_Sleep_Init();
    
    /* KEY 休眠初始化配置 */
    KEY_Sleep_Init();
    LED_Sleep_Init();
    
    /* ADC 休眠初始化配置 */
    ADC_Sleep_Init();
    
    /* GPTIM0 休眠初始化配置 */
    GPTIM0_Sleep_Init();
    
    /* GPTIM1 休眠初始化配置 */
    GPTIM1_Sleep_Init();
    
    /* IIC 休眠初始化配置 */
    I2C_Sleep_Init();
    
    /* SPI2 休眠初始化配置 */
    SPI2_Sleep_Init();
    
    /* 切换Rx引脚为GPIO, 使能中断唤醒 */
    GPIO_interrupt_init();
    
    /* 休眠模式下时钟源关闭，SVD无法实现数字滤波，禁止数字滤波 */
    FL_SVD_DisableDigitalFilter(SVD);
    /* 使能SVD唤醒功能，目的是监控电源 */
    SVD_Interrupt_Init(FL_DISABLE);    
}


/**
  * @brief  进入 Sleep
  * @param  void
  * @retval void
  */
void enter_sleep(void)
{
    FL_PMU_SleepInitTypeDef LPM_InitStruct = {FL_DISABLE};
    
    uint32_t tempWKFR;/* 唤醒标志 */
    
    g_awake_init=1;
    
    LPM_InitStruct.deepSleep          = FL_PMU_SLEEP_MODE_DEEP;
    LPM_InitStruct.LDOLowPowerMode    = FL_PMU_LDO_LPM_DISABLE;
    #if (MAIN_CLOCK == 16000000)    
    LPM_InitStruct.wakeupFrequency    = FL_PMU_RCHF_WAKEUP_FREQ_16MHZ;
    #elif (MAIN_CLOCK == 24000000)
    LPM_InitStruct.wakeupFrequency    = FL_PMU_RCHF_WAKEUP_FREQ_24MHZ;
    #else
    LPM_InitStruct.wakeupFrequency    = FL_PMU_RCHF_WAKEUP_FREQ_8MHZ;
    #endif
    LPM_InitStruct.wakeupDelay        = FL_PMU_WAKEUP_DELAY_2US;
    LPM_InitStruct.coreVoltageScaling = FL_DISABLE;
    FL_PMU_Sleep_Init(PMU, &LPM_InitStruct);

    FL_RCC_RCMF_Disable();               /* 关闭RCMF */
    FL_RMU_PDR_Enable(RMU);              /* 打开PDR */
    FL_RMU_BORPowerDown_Disable(RMU);    /* 关闭BOR，节省2uA功耗 */

    /* 睡眠电源模式*/
    FL_PMU_SetLowPowerMode(PMU, FL_PMU_POWER_MODE_SLEEP_OR_DEEPSLEEP);
    FL_IWDT_ReloadCounter(IWDT);
    __disable_irq();
    __WFI();                            /* 休眠 */
    FL_IWDT_ReloadCounter(IWDT);
    tempWKFR = PMU->WKFR;
    
    /* 必须第一时间切换时钟 */
    SelXTHFToPLL(&XTHF_InitParameter,&PLL_InitParameter);
    
    LED_Awake_Init(0);
        
    /* 本次唤醒非唤醒源处理,对未使用的唤醒源进行处理 */
    if ((0 == (tempWKFR & PMU_WKFR_UART2WKF_Msk)) &&
        (0 == (tempWKFR & PMU_WKFR_LPTWKF_Msk)))
    {
        LIN_AWAKE_Init();
    }
    
    if (!(tempWKFR & PMU_WKFR_IOWKF_Msk))
    {
        KEY_Awake_Init(0);
    }
    __enable_irq();
    
    LED_AWAKE_TEST();
    
    /* 非唤醒源 */
    ADC_Awake_Init(0);
    GPTIM0_Awake_Init(0);
    GPTIM1_Awake_Init(0);
    I2C_Awake_Init(0);
    SPI2_Awake_Init(0);
    
    /* SVD禁止中断 */
    SVD_Interrupt_DeInit();
    /* SVD恢复初始配置 */
    SVD_Init(SVD_MONTIOR_VDD, FL_SVD_WARNING_THRESHOLD_GROUP11, FL_SVD_REFERENCE_1P0V);
}


/**
  * @brief  GPIO中断
  * @note   唤醒后配置
  * @param  void
  * @retval void
  */
void GPIO_IRQHandler(void)
{
    
    uint32_t triggerEdge;
    uint32_t flag_EXTI;
    
    /* PB11引脚外部中断处理 */
    triggerEdge = FL_GPIO_GetTriggerEdge(GPIO, FL_GPIO_EXTI_LINE_6);
    flag_EXTI = FL_GPIO_IsActiveFlag_EXTI(GPIO, FL_GPIO_EXTI_LINE_6);
    
    if ((FL_GPIO_EXTI_TRIGGER_EDGE_DISABLE != triggerEdge) && (0x01 == flag_EXTI))
    {   
        /* 清除中断标识 */
        FL_GPIO_ClearFlag_EXTI(GPIO, FL_GPIO_EXTI_LINE_6);
        /* 唤醒处理 */
        KEY_Awake_Init(0);
        
    }
    
    if(GPIO->EXTIISR & LIN_RX_EXTI_LINE)
    {
        
        #if (AUTOBAUD == 1)
        /* 引脚重配置输入 */
        /*  关闭RX 设为高阻态 */
        LIN_UART_RX_GPIO->FCR &= ~LIN_UART_RX_PIN_FCR_MASK;
        LIN_UART_RX_GPIO->PUEN &= ~LIN_UART_RX_PIN;/*禁能上拉功能*/
        LIN_UART_RX_GPIO->INEN &= ~LIN_UART_RX_PIN;
            
        /*  使能捕获 */
        LIN_UART_CAPTURE_GPIO->FCR &= ~LIN_UART_CAPTURE_PIN_FCR_MASK;/*使能为数字功能*/
        LIN_UART_CAPTURE_GPIO->FCR |= LIN_UART_CAPTURE_PIN_FCR_DIGIT;
        LIN_UART_CAPTURE_GPIO->DFS |= LIN_UART_CAPTURE_PIN;/*使能数字重定向*/
        
        /* 清中断标志, 使能GPTIM2中断 */
        ATIM->ISR = LIN_ATIM_ISR_MASK;
        NVIC_ClearPendingIRQ(ATIM_IRQn);
        NVIC_EnableIRQ(ATIM_IRQn);
        
        /* 使能定时器 */
        ATIM->CR1 |= ATIM_CR1_CEN_Msk;
        #else
        /*  使能RX */
        LIN_UART_RX_GPIO->INEN &=  ~LIN_UART_RX_PIN;/*禁能输入使能*/
        LIN_UART_RX_GPIO->PUEN |= LIN_UART_RX_PIN;/*使能上拉功能*/
        LIN_UART_RX_GPIO->DFS |= LIN_UART_RX_PIN;/*使能数字重定向*/        
        LIN_UART_RX_GPIO->FCR &= ~LIN_UART_RX_PIN_FCR_MASK;/*使能为数字功能*/
        LIN_UART_RX_GPIO->FCR |= LIN_UART_RX_PIN_FCR_DIGIT;    
        
        /* 清除UART中断标志, 使能UART接收中断 */
        LIN_UART->ISR = LIN_UART->ISR;
        NVIC_ClearPendingIRQ(LIN_UART_IRQType);
        NVIC_EnableIRQ(LIN_UART_IRQType);
        
        /* 使能UART接收功能 */
        LIN_UART->CSR |= UART_CSR_RXEN_Msk;
        #endif
        
        /* lin唤醒后配置 */
        LIN_AWAKE_Init();
        
        /* 清中断标志, 关闭GPIO中断 */
        GPIO->EXTIISR = LIN_RX_EXTI_LINE;
        FL_GPIO_SetTriggerEdge(GPIO, LIN_RX_EXTI_LINE, FL_GPIO_EXTI_TRIGGER_EDGE_DISABLE); 
        NVIC_ClearPendingIRQ(GPIO_IRQn);
    }
}

/**
  * @brief  应用app sleep判断
  * @param  void
  * @retval 1: Sleep
  *         0: Wake
  */
uint8_t app_is_sleep(void)
{
    if (LIN_IS_Sleep() &&
        KEY_IS_Sleep() &&
        GPTIM0_IS_Sleep() &&
        GPTIM1_IS_Sleep() &&
        I2C_IS_Sleep() &&
        SPI2_IS_Sleep() &&
        LED_IS_Sleep()&&
        ADC_IS_Sleep() )
    { 
        return 1;
    }
    else
    {
        return 0;
    }
}

/**
  * @brief  lin Sleep
  * @param  void
  * @retval void
  */
void sleep(void)
{
    prepare_sleep();
    enter_sleep();
}
#endif  /* End #if (SLEEP_SUPPORT == 1) */

