/**    
  ******************************************************************************
  * @attention    
  * Copyright 2023 SHANGHAI FUDAN MICROELECTRONICS GROUP CO., LTD.(FUDAN MICRO.)
  *        
  * Redistribution and use in source and binary forms, with or without modification,
  * are permitted provided that the following conditions are met: 
  *    
  * 1. Redistributions of source code must retain the above copyright notice, 
  *    this list of conditions and the following disclaimer.
  *    
  * 2. Redistributions in binary form must reproduce the above copyright notice,
  *    this list of conditions and the following disclaimer in the documentation
  *    and/or other materials provided with the distribution.
  *    
  * 3. Neither the name of the copyright holder nor the names of its contributors 
  *    may be used to endorse or promote products derived from this software without
  *    specific prior written permission.
  *    
  * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS “AS IS”
  * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE   
  * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
  * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
  * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
  * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
  * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT   
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
  *******************************************************************************
  */
#include "spi.h"

#define TIMEOUT_VAL     (0xFFFFFFFFUL)


/**
  * @brief  SPI 初始化
  * @param  SPIx：模块选择
  * @retval void
  */
void MF_SPI_Init(SPI_Type* SPIx)
{
    FL_GPIO_InitTypeDef GPIO_InitStruct = {0};
    FL_SPI_InitTypeDef SPI_InitStruct = {0};    
    
    switch((uint32_t)SPIx)
    {
        case SPI1_BASE:
            
            GPIO_InitStruct.pin        = FL_GPIO_PIN_2;
            GPIO_InitStruct.mode       = FL_GPIO_MODE_DIGITAL;
            GPIO_InitStruct.outputType = FL_GPIO_OUTPUT_PUSHPULL;
            GPIO_InitStruct.pull       = FL_DISABLE;
            GPIO_InitStruct.remapPin   = FL_DISABLE;
            FL_GPIO_Init(GPIOD, &GPIO_InitStruct);
        
            GPIO_InitStruct.pin        = FL_GPIO_PIN_8 | FL_GPIO_PIN_9 | FL_GPIO_PIN_10 | FL_GPIO_PIN_11;
            GPIO_InitStruct.mode       = FL_GPIO_MODE_DIGITAL;
            GPIO_InitStruct.outputType = FL_GPIO_OUTPUT_PUSHPULL;
            GPIO_InitStruct.pull       = FL_DISABLE;
            GPIO_InitStruct.remapPin   = FL_DISABLE;
            (void)FL_GPIO_Init(GPIOB, &GPIO_InitStruct);
            break;
        
        case SPI2_BASE:
            GPIO_InitStruct.pin        = FL_GPIO_PIN_7 | FL_GPIO_PIN_8 | FL_GPIO_PIN_9 | FL_GPIO_PIN_10;
            GPIO_InitStruct.mode       = FL_GPIO_MODE_DIGITAL;
            GPIO_InitStruct.outputType = FL_GPIO_OUTPUT_PUSHPULL;
            GPIO_InitStruct.pull       = FL_DISABLE;
            GPIO_InitStruct.remapPin   = FL_DISABLE;
            (void)FL_GPIO_Init(GPIOC, &GPIO_InitStruct);
            break;
    }
    
    /*通信模式选择配置*/
    SPI_InitStruct.transferMode  = FL_SPI_TRANSFER_MODE_FULL_DUPLEX;
    /*Master/Slave模式选择*/
    SPI_InitStruct.mode          = FL_SPI_WORK_MODE_MASTER;
    /*数据位宽配置*/
    SPI_InitStruct.dataWidth     = FL_SPI_DATA_WIDTH_8B;
    /*时钟极性选择配置*/
    SPI_InitStruct.clockPolarity = FL_SPI_POLARITY_INVERT;
    /*时钟相位选择配置*/
    SPI_InitStruct.clockPhase    = FL_SPI_PHASE_EDGE2;
    /*SSN输出控制*/
    SPI_InitStruct.softControl   = FL_ENABLE;
    /*Master模式波特率配置*/
    SPI_InitStruct.baudRate      = SPI2_BAUD;
    /*帧格式配置*/
    SPI_InitStruct.bitOrder      = FL_SPI_BIT_ORDER_MSB_FIRST;
    
    FL_SPI_Init(SPIx, &SPI_InitStruct);
}


/**
  * @brief  SPI DMA 初始化
  * @param  SPIx：模块选择
  * @retval void
  */
void SPI_DMA_Init(SPI_Type* SPIx)
{
    FL_DMA_InitTypeDef DMA_InitStruct = {0};
    
    MF_SPI_Init(SPIx);
    
    DMA_InitStruct.periphAddress        = FL_DMA_PERIPHERAL_FUNCTION1;
    DMA_InitStruct.direction            = FL_DMA_DIR_PERIPHERAL_TO_RAM;
    DMA_InitStruct.memoryAddressIncMode = FL_DMA_MEMORY_INC_MODE_INCREASE;
    DMA_InitStruct.flashAddressIncMode  = FL_DMA_FLASH_INC_MODE_INCREASE;
    DMA_InitStruct.dataSize = FL_DMA_BANDWIDTH_8B;
    DMA_InitStruct.priority = FL_DMA_PRIORITY_LOW;
    DMA_InitStruct.circMode = FL_DISABLE;

    FL_DMA_Init(DMA, &DMA_InitStruct, FL_DMA_CHANNEL_5);

    DMA_InitStruct.periphAddress        = FL_DMA_PERIPHERAL_FUNCTION1;
    DMA_InitStruct.direction            = FL_DMA_DIR_RAM_TO_PERIPHERAL;
    DMA_InitStruct.memoryAddressIncMode = FL_DMA_MEMORY_INC_MODE_INCREASE;
    DMA_InitStruct.flashAddressIncMode  = FL_DMA_FLASH_INC_MODE_INCREASE;
    DMA_InitStruct.dataSize = FL_DMA_BANDWIDTH_8B;
    DMA_InitStruct.priority = FL_DMA_PRIORITY_LOW;
    DMA_InitStruct.circMode = FL_DISABLE;

    FL_DMA_Init(DMA, &DMA_InitStruct, FL_DMA_CHANNEL_6);
    
    FL_DMA_Enable(DMA);
}



/**
  * @brief  SPI读写数据
  * @param  data: 写数据
  * @retval data: 读数据
  */
uint32_t SpiWriteAndRead(SPI_Type* SPIx, uint32_t data)
{
    uint32_t timeout = 0;

    FL_SPI_SetSSNPin(SPIx, FL_SPI_SSN_LOW);

    FL_SPI_WriteTXBuff(SPIx, data);

    do
    {
        timeout++;
    } while(!FL_SPI_IsActiveFlag_TXBuffEmpty(SPIx) && timeout < TIMEOUT_VAL);

    timeout = 0;
    do
    {
        timeout++;
    } while(!FL_SPI_IsActiveFlag_RXBuffFull(SPIx) && timeout < TIMEOUT_VAL);

    data = FL_SPI_ReadRXBuff(SPIx);

    FL_SPI_SetSSNPin(SPIx, FL_SPI_SSN_HIGH);

    return data;
}



/**
  * @brief  SPI写数据
  * @param  data:   数据指针
  *         length: 数据长度
  * @retval void
  */
void SpiWriteData(SPI_Type* SPIx, uint8_t *data, uint16_t length)
{
    uint32_t timeout;
   FL_SPI_SetSSNPin(SPIx, FL_SPI_SSN_LOW);

    while(length--)
    {
        FL_SPI_WriteTXBuff(SPIx, *(data++));

        /* 等待发送缓存空 */ 
        timeout = 0;
        do
        {
            timeout++;
        } while(!FL_SPI_IsActiveFlag_TXBuffEmpty(SPIx) && timeout < TIMEOUT_VAL);

        timeout = 0;
        do
        {
            timeout++;
        } while(!FL_SPI_IsActiveFlag_RXBuffFull(SPIx) && timeout < TIMEOUT_VAL);
        
        FL_SPI_ReadRXBuff(SPIx);
    }
     FL_SPI_SetSSNPin(SPIx, FL_SPI_SSN_HIGH);

}

/**
  * @brief  SPI读数据
  * @param  data:   数据指针
  *         length: 数据长度
  * @retval void
  */
void SpiReadData(SPI_Type* SPIx, uint8_t *data, uint16_t length)
{
    uint32_t timeout;

    FL_SPI_SetSSNPin(SPIx, FL_SPI_SSN_LOW);

    while(length--)
    {
        FL_SPI_WriteTXBuff(SPIx, 0x55);
        timeout = 0;
        do
        {
            timeout++;
        } while(!FL_SPI_IsActiveFlag_RXBuffFull(SPIx) && timeout < TIMEOUT_VAL);

        *data = FL_SPI_ReadRXBuff(SPIx);
        data++;
    }

    FL_SPI_SetSSNPin(SPIx, FL_SPI_SSN_HIGH);

}


/**
  * @brief  SPI DMA读写数据
  * @param  wdata：   写数据地址
            rdata:    读数据地址
  *         length:   数据长度
  * @retval void
  */
void SPI_ReadWrite_DMA( SPI_Type* SPIx , uint8_t *wdata, uint8_t* rdata, uint16_t length)
{
    FL_DMA_ConfigTypeDef configStruct;

    FL_SPI_SetSSNPin(SPIx, FL_SPI_SSN_LOW);
    
    configStruct.memoryAddress     = (uint32_t)wdata;
    configStruct.transmissionCount = length - 1;
    FL_DMA_StartTransmission(DMA, &configStruct, FL_DMA_CHANNEL_6);
    configStruct.memoryAddress     = (uint32_t)rdata;
    configStruct.transmissionCount = length - 1;
    FL_DMA_StartTransmission(DMA, &configStruct, FL_DMA_CHANNEL_5);

    while(!FL_DMA_IsActiveFlag_TransferComplete(DMA, FL_DMA_CHANNEL_5));
    while(!FL_DMA_IsActiveFlag_TransferComplete(DMA, FL_DMA_CHANNEL_6));
    while(FL_SPI_IsActiveFlag_Busy(SPIx));

    FL_SPI_SetSSNPin(SPIx, FL_SPI_SSN_HIGH);
}


/**
  * @brief  SPI 测试示例
  * @param  void
  * @retval void
  */
void SPI_Test(void)
{
    uint8_t wbuf[8] = {0x11, 0x22, 0x33, 0x44, 0x55, 0x66, 0xAA, 0xBB};
    uint8_t rbuf[8] = {0};

    /* Read and write */ 
    SPI_ReadWrite_DMA(SPI2, wbuf, rbuf, 8);
}

/**
  * @brief  SPI1模块复位函数
  * @param  void
  * @retval void
  */
void SPI1_DeInit(void)
{
    /* 恢复外设寄存器到复位值 */
    FL_SPI_DeInit(SPI1);

    /* 清除中断挂起 */
    NVIC_ClearPendingIRQ(SPI1_IRQn);
    
    /* 关闭中断 */
    NVIC_DisableIRQ(SPI1_IRQn);
}

/**
  * @brief  SPI2模块复位函数
  * @param  void
  * @retval void
  */
void SPI2_DeInit(void)
{
    /* 恢复外设寄存器到复位值 */
    FL_SPI_DeInit(SPI2);

    /* 清除中断挂起 */
    NVIC_ClearPendingIRQ(SPI2_IRQn);
    
    /* 关闭中断 */
    NVIC_DisableIRQ(SPI2_IRQn);
}


/* 睡眠支持 */
#if (SLEEP_SUPPORT == 1)
/**
  * @brief  SPI2 休眠前初始化
  * @param  void
  * @retval void
  */
void SPI2_Sleep_Init(void)
{
    /* 关闭非唤醒外设 */
    FL_SPI_Disable(SPI2);
    
    /* 关闭非唤醒IO口 */
    CloseIO(GPIOC, FL_GPIO_PIN_7);
    CloseIO(GPIOC, FL_GPIO_PIN_8);
    CloseIO(GPIOC, FL_GPIO_PIN_9);
    CloseIO(GPIOC, FL_GPIO_PIN_10);
    
    /* 唤醒源配置 */
}

/**
  * @brief  SPI2 唤醒后初始化
  * @param  0-普通 1-作为本次唤醒的源
  * @retval void
  */
void SPI2_Awake_Init(uint8_t awake_type)
{
    /* 打开非唤醒IO */
    AltFunIO( GPIOC, FL_GPIO_PIN_7,  FL_GPIO_OUTPUT_PUSHPULL, FL_DISABLE, FL_DISABLE );      /* PC12 --> SPI2_SSN */
    AltFunIO( GPIOC, FL_GPIO_PIN_8,  FL_GPIO_OUTPUT_PUSHPULL, FL_DISABLE, FL_DISABLE );      /* PC8 --> SPI2_SCK */
    AltFunIO( GPIOC, FL_GPIO_PIN_9,  FL_GPIO_OUTPUT_PUSHPULL, FL_DISABLE, FL_DISABLE );      /* PC9 --> SPI2_MISO */
    AltFunIO( GPIOC, FL_GPIO_PIN_10, FL_GPIO_OUTPUT_PUSHPULL, FL_DISABLE, FL_DISABLE );      /* PC10 --> SPI2_MOSI */
    
    /* 任务初始化 */

    /* 唤醒源处理 */
    
    /* 非唤醒外设处理 */
    FL_SPI_Enable(SPI2);  
}

/**
  * @brief  SPI2 sleep判断
  * @param  void
  * @retval 1: Sleep
  *         0: Wake
  */
uint8_t SPI2_IS_Sleep(void)
{
    if (!FL_SPI_IsActiveFlag_Busy(SPI2))
    {
        return 1;
    }
    else
    {
        return 0;
    }
}
#endif

