/**    
  ******************************************************************************
  * @attention    
  * Copyright 2023 SHANGHAI FUDAN MICROELECTRONICS GROUP CO., LTD.(FUDAN MICRO.)
  *        
  * Redistribution and use in source and binary forms, with or without modification,
  * are permitted provided that the following conditions are met: 
  *    
  * 1. Redistributions of source code must retain the above copyright notice, 
  *    this list of conditions and the following disclaimer.
  *    
  * 2. Redistributions in binary form must reproduce the above copyright notice,
  *    this list of conditions and the following disclaimer in the documentation
  *    and/or other materials provided with the distribution.
  *    
  * 3. Neither the name of the copyright holder nor the names of its contributors 
  *    may be used to endorse or promote products derived from this software without
  *    specific prior written permission.
  *    
  * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS “AS IS”
  * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE   
  * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
  * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
  * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
  * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
  * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT   
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
  *******************************************************************************
  */
#include "user_init.h"
#include "uart.h"
#include "lin.h"

/**
  * @brief  将RCHF配置为系统时钟源
  *         当系统时钟频率超过32MHz时，必须使能预取指令
  *         当系统时钟频率超过8NHz时，访问跨电源接口(CDIF)外设时钟必须分频
  * @param  clock: RCHF振荡频率值
  * @retval void
  */
void ClockInit(uint32_t clock)
{
    FL_FLASH_SetReadWait(FLASH, FL_FLASH_READ_WAIT_2CYCLE);
    switch (clock)
    {
    case FL_RCC_RCHF_FREQUENCY_8MHZ:
        FL_RCC_RCHF_WriteTrimValue(RCHF8M_TRIM);
        FL_FLASH_SetReadWait(FLASH, FL_FLASH_READ_WAIT_0CYCLE);
        break;

    case FL_RCC_RCHF_FREQUENCY_16MHZ:
        FL_RCC_RCHF_WriteTrimValue(RCHF16M_TRIM);
        FL_FLASH_SetReadWait(FLASH, FL_FLASH_READ_WAIT_1CYCLE);
        break;

    case FL_RCC_RCHF_FREQUENCY_24MHZ:
        FL_RCC_RCHF_WriteTrimValue(RCHF24M_TRIM);
        FL_FLASH_SetReadWait(FLASH, FL_FLASH_READ_WAIT_1CYCLE);
        break;

    default:
        FL_RCC_RCHF_WriteTrimValue(RCHF8M_TRIM);
        FL_FLASH_SetReadWait(FLASH, FL_FLASH_READ_WAIT_0CYCLE);
        break;
    }

    FL_RCC_RCHF_SetFrequency(clock);
    FL_RCC_SetSystemClockSource(FL_RCC_SYSTEM_CLK_SOURCE_RCHF);
}


#ifndef MFANG

#ifdef __CC_ARM
#pragma import(__use_no_semihosting)
/* 标准库需要的支持函数 */
struct __FILE
{
    int handle;
};
FILE __stdout;
#endif

/* 定义_sys_exit()以避免使用半主机模式 */
void _sys_exit(int x)
{
    x = x;
}

/* 重定义fputc函数 */
int fputc(int ch, FILE *f)
{
    FL_UART_WriteTXBuff(UART5, (uint8_t)ch);
    while (FL_UART_IsActiveFlag_TXBuffEmpty(UART5) != FL_SET);
    return ch;
}

/**
  * @brief  调试串口初始化
  * @param  void
  * @retval void
  */
void DebugUartInit(void)
{
    FL_GPIO_InitTypeDef GPIO_InitStruct = {0};
    FL_UART_InitTypeDef UART_InitStruct = {0};

    /* PD0:UART5-RX   PD1:UART5-TX */
    GPIO_InitStruct.pin        = FL_GPIO_PIN_0 | FL_GPIO_PIN_1;
    GPIO_InitStruct.mode       = FL_GPIO_MODE_DIGITAL;
    GPIO_InitStruct.outputType = FL_GPIO_OUTPUT_PUSHPULL;
    GPIO_InitStruct.pull       = FL_ENABLE;
    GPIO_InitStruct.remapPin = FL_DISABLE;
    FL_GPIO_Init(GPIOD, &GPIO_InitStruct);

    UART_InitStruct.clockSrc = FL_RCC_UART0_CLK_SOURCE_APB1CLK;

    UART_InitStruct.baudRate            = 115200;                                   /* 波特率 */
    UART_InitStruct.dataWidth           = FL_UART_DATA_WIDTH_8B;                    /* 数据位数 */
    UART_InitStruct.stopBits            = FL_UART_STOP_BIT_WIDTH_1B;                /* 停止位 */
    UART_InitStruct.parity              = FL_UART_PARITY_EVEN;                      /* 奇偶校验 */
    UART_InitStruct.transferDirection   = FL_UART_DIRECTION_TX_RX;                  /* 接收-发送使能 */
    FL_UART_Init(UART5, &UART_InitStruct);
}

#endif

/**
  * @brief  更改系统主频为RCHF基准的PLL
  * @param  void
  * @retval void
  */
void Clockchange(void)
{
    /* PLL配置 */
    FL_RCC_PLL_SetClockSource(FL_RCC_PLL_CLK_SOURCE_RCHF);
    FL_RCC_PLL_SetPrescaler(FL_RCC_PLL_PSC_DIV8); /* 8分频为1M */
    FL_RCC_PLL_WriteMultiplier(0x17); /* 倍频24M */
    FL_RCC_PLL_Enable();

    while(!FL_RCC_IsActiveFlag_PLLReady());  /* 等待建立 */

    FL_RCC_SetSystemClockSource(FL_RCC_SYSTEM_CLK_SOURCE_PLL);
    FL_RCC_SetAHBPrescaler(FL_RCC_AHBCLK_PSC_DIV1);
    FL_RCC_SetAPB1Prescaler(FL_RCC_APB1CLK_PSC_DIV1);
    FL_RCC_SetAPB2Prescaler(FL_RCC_APB2CLK_PSC_DIV1);
}


/**
  * @brief  用户参数配置
  * @param  void
  * @retval void
  */
void UserInit(void)
{
#ifndef MFANG
    #if (MAIN_CLOCK == 16000000)
    ClockInit(FL_RCC_RCHF_FREQUENCY_16MHZ);
    #elif (MAIN_CLOCK == 24000000)
    ClockInit(FL_RCC_RCHF_FREQUENCY_24MHZ);
    #endif
    /* Update System Core Clock */
    SystemCoreClockUpdate();    
//    DebugUartInit();
#endif
}
